/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This is the main loop for the Rogus Application Framework. It lists all of the
// parsers which have been registered using a ParserRegistration object, and allows
// the user to choose one.

#include "ParserRegistration.h"
#include "Rogus_McBogus.h"
#include <stdio.h>

//--------------------------------------------------------------------------------------------------

#pragma warning(disable: 4127)
// warning C4127: conditional expression is constant (for while(True) below)

void
main(int argc, char *argv[])
{
	char buf[80];
	try
	{
      rogus::user_prefs up;
      if (argc==3)
      {
         strcpy(up.default_input,  argv[1]);
         strcpy(up.default_output, argv[2]);
      }
      rogus::Rogus_McBogus rog(up);
		while (true)
		{
			Sleep(100); // give text window time to catch up
			printf("\nPlease select a parser by choosing a number from below:\n");
			printf("\t 0. exit\n");
			int i;
			int numparsers = ParserRegistration::getNumParsers();
			for (i = 0; i < numparsers; i++)
				printf("\t%2d. %s\n",i+1, ParserRegistration::getParserName(i));
			fgets(buf, 79, stdin);
			int choice = atoi(buf);
			if (choice < 1)
				break;
			choice--;
			if (choice >= numparsers)
			{
				printf("Not a valid choice.\n");
				continue;
			}
			rog.setParser(ParserRegistration::getParserFunc(choice)(&rog),
                           ParserRegistration::getParserName(choice));
			Sleep(300); // give text window time to catch up
			printf("To choose a new parser, press return.\n");
			fgets(buf, 79, stdin);
         rog.setParser((rogus::parser*)NULL, "");
			rog.o.closeAllInputs(); // close old parser's inputs
//			oap->closeAllOutputs(); // close old parser's outputs
		}
	}
   catch(rogus::Rogus_McBogus::exception)
	{
		printf("Rogus McBogus terminated abnormally. Press return to continue\n");
		fgets(buf, 79, stdin);
	}
}

#pragma warning(default : 4127)

// Authors: Ben Denckla, Patrick Pelletier
