/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser is an example of how to use the Rogus score player. The MIDI file
// "test_MIDI_file.mid" from the current directory is read in, and each key on the
// input device is mapped to play this score at a different speed and transposition.

#include <math.h>
#include "Transposer.h"
#include "NotePairScorePlayer.h"
#include "ParserRegistration.h"
#include "Filedata.h"
#include "Rogus_McBogus.h"
#include "NotePairScore.h"
#include "NotePairScorePlayer.h"

using namespace rogus;

class NotePairScoreParser : public parser
{
	typedef UtilPtr<NotePairScore> NotePairScorePtr;
public:
	NotePairScoreParser( NotePairScorePtr );
	~NotePairScoreParser();
};

static NotePairScorePlayer* sp_array[128];
static Rogus_McBogus* rog;

static
double
log2(double x)
{
	return (log(x)/log(double(2)));
}

static
void
my_output_function(const NotePairScoreElement& npse, NotePairScoreScale scale, ScheduleTime now)
{
	// extract transposition information from scale
	int t = -int(log2(scale)*12 + 0.5);
	// (actually, there shouldn't be any note-offs, since they're all paired up)
	if (npse.msg.msgt() == MidiMsg::note_on || npse.msg.msgt() == MidiMsg::note_off)
	{
		NotePairScoreElement mynpse = npse;
		int key = mynpse.msg.key();
		key += t;
		// do octave wrapping
		while (key < 0)
			key += 12;
		while (key > 127)
			key -= 12;
		mynpse.msg.set_key(key);
		mynpse.play(scale, now, rog);
	}
	else
		npse.play(scale, now, rog);
}

static
void
note_on( MidiMsg *m ) 
{
	if (m->d2 == 0)
		sp_array[m->d1]->stop();
	else
		sp_array[m->d1]->start();
}

static
void
note_off( MidiMsg *m )
{
	sp_array[m->d1]->stop();
}

NotePairScoreParser::
NotePairScoreParser( NotePairScorePtr myNotePairScore )
{
	rog->o.open_input(); // open the default input
	
	for (int i = 0; i < 128; i++)
	{
		sp_array[i] = new NotePairScorePlayer(rog, myNotePairScore, my_output_function);
//		sp_array[i]->addFilter(new Transposer(i-60));
		sp_array[i]->setScale(NotePairScoreScale(pow(double(2), (double(60-i))/12)));
		sp_array[i]->setLoop(true);
	}

	msg_hnd[MidiMsg::note_on] = note_on;
	msg_hnd[MidiMsg::note_off] = note_off;
}

NotePairScoreParser::
~NotePairScoreParser()
{
	int i;
	for (i = 0; i < 128; i++)
	{
		sp_array[i]->stop();
		delete sp_array[i];
	}
}

static parser* newFunc(Rogus_McBogus *r)
{
   rog = r;
	FiledataPtr fdp = new Filedata("test_MIDI_file.mid",&rog->ew);
	ScoreCreationInfo sci(rog,fdp);
	sci.addRemainingTracks(rog->o.open_output());
	return (new NotePairScoreParser(new NotePairScore(&sci)));
}

static ParserRegistration reg(newFunc, "NotePairScoreParser");

// Authors: Ben Denckla, Patrick Pelletier
