/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser is meant to be used with the default output looped back
// to the default input. It sends out MIDI messages and measures the amount
// of time it takes for them to get back

#include "ParserRegistration.h"
#include "Rogus_McBogus.h"

using namespace rogus;

class PingParser : public parser
{
public:
	PingParser();
	~PingParser();
};

static bool running;
static ioref ou_node;
static Rogus_McBogus *r;

static int count;
static ScheduleTime lastsent;

static void infunc(MidiMsg*)
{
	if (lastsent == NEVER)
		r->tw.pr("extraneous message!\n");
	ScheduleTime t = r->o.time();
	r->tw.pr("message: %d time: %d demimilliseconds\n", count++, t-lastsent);
	lastsent = NEVER;
}

static ScheduleTime func(void*)
{
	if (!running)
		return NEVER;
	if (lastsent != NEVER)
		r->tw.pr("Dropped message %d\n",count++);
	ScheduleTime t = r->o.time();
	MidiMsg m;
	m.set(
      (unsigned char)
		(
         ((count/2)%2) ? 
         MidiMsg::note_off :
         MidiMsg::note_on
		),
		0, uchar(60+(count%2)), 64, ou_node);

	r->o.write(&m);
	lastsent = t;
	return 2000;
}

PingParser::
PingParser()
{
	msg_hnd[MidiMsg::note_on] = infunc;
	msg_hnd[MidiMsg::note_off] = infunc;
	ou_node = r->o.open_output(); // open the default output
	r->o.open_input(); // open the default input
	running = true;
	count = 0;
	lastsent = NEVER;
	task_data td(func, NULL, r->o.time()+2000);
	r->s.insert(td);
}

PingParser::
~PingParser()
{
	running = false;
}

static parser* newFunc(Rogus_McBogus *in_r)
{
   r = in_r;
	return (new PingParser());
}

static ParserRegistration reg(newFunc, "PingParser");

// Author: Patrick Pelletier
