/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

/* This file gives example of how to create a parser with hooks into your own MIDI 
input processing functions.  In this case, a delayed octave generator is made. */


#include <stdio.h>
#include "Rogus_McBogus.h"
#include "data_structs.h"

using namespace rogus;

static Rogus_McBogus *r;
static ioref ou_node;
static pool<MidiMsg> b(50, "");
static bool stop[128];

static const int note_len = 250, delay = 150;

static long wake( void *arg )
{
	MidiMsg* m = (MidiMsg*) arg;
	if (stop[m->key()])
	{
		b.release(m);
		return NEVER;
	}
	else
	{
		r->s.play_note( *m, note_len );
		return delay;
	}
}

static void note_on( MidiMsg *m ) 
{
	m->node = ou_node;
	if (m->vel())
	{
		r->s.play_note( *m, note_len );
		
		stop[m->key()] = false;

		MidiMsg *n = b.allocate();
		if (!n)
			r->ew.w( "allocate failed" );
		else
		{
			*n = *m;

			task_data td( wake, n, m->time + delay );
			int e = r->s.insert( td );
			if (e) r->ew.w( "insert failed in octave parser" );
		}
	}
	else stop[m->key()] = true;
}

class octave_parser : public parser
{
public:
	octave_parser(Rogus_McBogus*);
};

octave_parser::
octave_parser(Rogus_McBogus *in_r)
{
   r = in_r;
	ou_node = r->o.open_output(); // open the default output, and save its ioref
   r->o.open_input(); // open the default input

	msg_hnd[MidiMsg::note_on] = note_on;
}


//--------------------------------------------------------------------------------------------------

void
main(int argc, char *argv[])
{
	char buf[80];
	try
	{
      user_prefs up;
      if (argc==3)
      {
         strcpy(up.default_input,  argv[1]);
         strcpy(up.default_output, argv[2]);
      }
		Rogus_McBogus rogus(up);
		Sleep(100); // give text window time to catch up

      
      parserPtr p = new octave_parser(&rogus);
		rogus.setParser(p,"octave_parser");

		Sleep(300); // give text window time to catch up
		printf("To exit, press return.\n");
		fgets(buf, 79, stdin);
	}
   catch(Rogus_McBogus::exception)
	{
		printf("Rogus McBogus terminated abnormally. Press return to continue\n");
		fgets(buf, 79, stdin);
	}
}

// Authors: Ben Denckla, Patrick Pelletier
