/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#pragma warning(disable : 4514)
// this is the warning for "unreferenced inline function has been removed"

#include "BinStatistic.h"
#include "string.h"
namespace rogus
{

BinStatistic::BinStatistic(int n, int (*func)(int), const char* nam) :
   bins(n)
{
	name = nam;
	for (int i = 0; i < n; i++)
		bins[n-1-i].min = func(i);
	clear();
}

void BinStatistic::clear()
{
	for (uint4 i = 0; i < bins.length; i++)
		bins[i].value = 0;
	max = 0;
}

void BinStatistic::stat(FILE* f)
{
	fprintf(f, "%s:\n", name);
	double sum = 0;
	uint4 i;
	for (i = 0; i < bins.length; i++)
		sum += bins[i].value;
	double avg = sum/bins.length;
	double multiplier = 20/avg;
	char** bufs = new char*[bins.length];
	unsigned int maxlen = 0;
	for (i = 0; i < bins.length; i++)
	{
		char* s = new char[80];
		if (i > 0 && bins[i].min == bins[i-1].min-1)
			sprintf(s, "  %d ", bins[i].min);
		else if (i > 0)
			sprintf(s, "  %d - %d ", bins[i].min, bins[i-1].min-1);
		else
			sprintf(s, "  %d & up ", bins[i].min);
		bufs[i] = s;
		if (maxlen < strlen(s))
			maxlen = strlen(s);
	}
	for (i = bins.length; i > 0; i--)
	{
		int j = i-1;
		char* s = bufs[j];
		for (unsigned int K = 0; K < maxlen - strlen(s); K++)
			fprintf(f, " ");
		fprintf(f,"%s |",s);
		delete[] s;
		int howFar = (int) (multiplier*bins[j].value + 0.5);
		bool overflow = 0;
		if (howFar+maxlen > 70)
		{
			howFar = 70-maxlen;
			overflow = 1;
		}
		for (int k = 0; k < howFar; k++)
			fprintf(f,"*");
		if (overflow)
			fprintf(f,"+");
		if (bins[j].value > 0)
			fprintf(f,"   %d\n", bins[j].value);
		else
			fprintf(f,"\n");
	}
	fprintf(f, "  (maximum = %d)\n\n", max);
	delete[] bufs;
}

void BinStatistic::throwInBin(int n)
{
	if (n > max)
		max = n;
	for (uint4 i = 0; i < bins.length; i++)
		if (n >= bins[i].min)
		{
			bins[i].value++;
			break;
		}
}
}
// Author: Patrick Pelletier

