/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "Rogus_McBogus.h"
#include "Rogus.h"
#include "MIDIsvc.h"
#include "text_win.h"
#include "err_and_warn.h"
#include "sched.h"

//--------------------------------------------------------------------------------------------------
namespace rogus
{


bool Rogus_McBogus::rogus_exists = false;

Rogus_McBogus::
Rogus_McBogus(user_prefs ip) : up(ip), ew(&tw), s(&up,&cs,&ew), o(&up,&s,&cs,&ew)
{
   if (rogus_exists == true)
   {
		ew.e("You can't make more than one Rogus at a time!");
      throw rx();
   }

   tw.init(up.rmfp);
   s.set_oap(&o); // 1
   
   ew.i("This is Rogus McBogus version %s for Windows.",version());
	rogus_exists = true;
}

/*
1. This is a hack caused by the interdependency of MIDIsvc and scheduler.
scheduler really needs MIDIsvc as an argument to its constructor, but since
MIDIsvc needs scheduler as an argument to its constructor, we are in a bind.
It is this bind which leads to the "late construction" of scheduler using
set_oap().
*/

Rogus_McBogus::
~Rogus_McBogus()
{
	setParser((parser*)NULL, "");
	rogus_exists = false;
}

void
Rogus_McBogus::
setParser(parserPtr pp, const char* name)
{
	if (p != NULL && up.statistics_on)
		Statistic::report(version(), &ew);
   o.lp = NULL;
	p = pp;
   o.lp = pp;
	if (pp != NULL)
	{
		Statistic::reset(name);
		ew.i("%s now active.",name);
	}
}


#ifdef NDEBUG
#define APPEND_TO_VERSION ""
#else
#define APPEND_TO_VERSION " DEBUG"
#endif

const char*
Rogus_McBogus::
version()
{
   static const char* s = "1.9a" APPEND_TO_VERSION;
   return s;
}
}
// Authors: Ben Denckla, Patrick Pelletier
