/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "Statistic.h"
#include "Rogus.h"
#include "text_win.h"
namespace rogus
{


static time_t startTime;
static const char* parserName = "unknown";

Statistic* Statistic::first = NULL;

static const char* myAsctime(struct tm* t)
{
	static char buf[80];
	strcpy(buf,asctime(t));
	for (unsigned int i = 0; i < strlen(buf); i++)
		if (buf[i] < ' ')
			buf[i] = 0;
	return buf;
}

void Statistic::reset(const char* s)
{
	for (Statistic* p = first; p != NULL; p = p->next)
		p->clear();
	startTime = time(NULL);
	parserName = s;
}

void Statistic::report(const char *version, err_and_warn* ewp)
{
	// find out name of machine
	char machine_name[MAX_COMPUTERNAME_LENGTH + 1];
	DWORD machine_name_size = MAX_COMPUTERNAME_LENGTH + 1;
	strcpy(machine_name, "unknown");
	if (!GetComputerName(machine_name, &machine_name_size))
	{
		ewp->w("GetComputerName failed");
		strcpy(machine_name, "unknown");
	}
	// find out name of OS
	char os_name[40+128];
	OSVERSIONINFO ovi;
	ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&ovi))
	{
		ewp->w("GetVersionEx failed");
		strcpy(os_name, "unknown");
	}
	else
	{
		char extra_info[40+128];
		if (strlen(ovi.szCSDVersion) > 0)
			sprintf(extra_info," [%s]",ovi.szCSDVersion);
		else
			extra_info[0] = 0;
		if (ovi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
			sprintf(os_name, "Windows 95 %d.%d build %d%s", (int)ovi.dwMajorVersion, (int)ovi.dwMinorVersion, (int)(ovi.dwBuildNumber & 0xffff), extra_info);
		else if (ovi.dwPlatformId == VER_PLATFORM_WIN32_NT)
			sprintf(os_name, "Windows NT %d.%d build %d%s", (int)ovi.dwMajorVersion, (int)ovi.dwMinorVersion, (int)(ovi.dwBuildNumber & 0xffff), extra_info);
		else
			strcpy(os_name, "unknown");
	}
	// find out name of user
	char user_name[40];
	DWORD user_name_size = 40;
	strcpy(user_name, "unknown");
	if (!GetUserName(user_name, &user_name_size))
	{
		ewp->w("GetUserName failed");
		strcpy(user_name, "unknown");
	}
	// write out the header to the log file
	const char* logfile = "Rogus.log";
	FILE* f = fopen(logfile, "a");
	if (f == NULL)
	{
		ewp->w("Unable to write \"%s\"", logfile);
		return;
	}
	fprintf(f,"Rogus McBogus %s running %s\n",version,parserName);
	fprintf(f,"on %s (%s) by %s\n",machine_name, os_name, user_name);
	struct tm* t = localtime(&startTime);
	fprintf(f,"%s to ",myAsctime(t));
	time_t stopTime = time(NULL);
	t = localtime(&stopTime);
	fprintf(f,"%s\n\n",myAsctime(t));
	// go through and report on each statistic
	for (Statistic* p = first; p != NULL; p = p->next)
		p->stat(f);
	for (int i = 0; i < 78; i++)
		fprintf(f,"-");
	fprintf(f,"\n");
	fclose(f);
}

Statistic::Statistic()
{
	prev = NULL;
	next = first;
	first = this;
	if (next != NULL)
		next->prev = this;
}

Statistic::~Statistic()
{
	if (next != NULL)
	next->prev = prev;
	if (prev != NULL)
		prev->next = next;
	else
		first = next;
}
}
// Author: Patrick Pelletier
