/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#pragma warning(disable : 4514)
// this is the warning for "unreferenced inline function has been removed"

#include "MidiMsg.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>

namespace rogus
{

int
MidiMsg::
note_num(const char* name)
{
	static const int lookup[] = { 0, 2, 4, 5, 7, 9, 11 };
	int len;
	if (name == NULL || (len = strlen(name)) < 2)
		return -1;
	int num = tolower(name[0]);
	if (num < 'a' || num > 'g')
		return -1;
	num -= 'c';
	if (num < 0)
		num += 7;
	num = lookup[num];
	num += 60;
	int i;
	for (i = 1; i < len -1; i++)
	{
		switch (name[i])
		{
		case 'b':
			num--;
			break;
		case '#':
			num++;
			break;
		default:
			return -1;
		}
	}
	if (name[len-1] < '0' || name[len-1] > '9')
		return -1;
	num += (12 * ((name[len-1] - '0') - 4));
	while (num < 0) // very unlikely, but could happen with a lot of flats
		num += 12;
	while (num > 127)
		num -= 12;
	return num;
}

int
MidiMsg::
len() const
{											//	0	1	2	3	4	5	6	7
	const char     msg_len[ 7] = {	3, 3, 3, 3, 2, 2, 3 };//8	9	A	B	C	D	E	F
	const char sys_msg_len[16] = {	0, 2, 3, 2, 0, 0, 1, 1,	1, 0, 1, 1, 1, 0, 1, 1 };

	int retval;
	int t = msgt();
	if ( t == MidiMsg::system )
	{
		int st = smsgt();
		retval = ( st == smsgt::sys_ex ? ed->size() : sys_msg_len[st] );
	}
	else retval = msg_len[t];
	
	return retval;
}
// Returns the length of the MidiMsg, with 0 indicating an undefined length.

bool MidiMsg::is_meta(int meta_type) const
{
	return (is_ext() && ed->st == smsgt::file_meta && ed->meta_event.mt == meta_type);
}

void
MidiMsg::
set_sys_ex( const uchar *r, long len )
{
	set_stat_byt( MidiMsg::system, smsgt::ext );
	ed = new ext_data( r, len );
}

void
MidiMsg::
set_meta( int meta_ty, const uchar *r, long len  )
{
	set_stat_byt( MidiMsg::system, smsgt::ext );
	ed = new ext_data( meta_ty, r, len );
}

void
MidiMsg::
sprint( char *str ) const
{
	int len = 0;
	len += sprintf( str+len, "%8ld ", time );
	if ( msgt() == MidiMsg::system && smsgt() == smsgt::ext )
	{
		len += sprintf( str+len, "ed: " );
		len += ed->sprint( str+len );
	}
	else len += sprintf( str+len, "%X %X %X ", s, d1, d2 );

}

UtilStringPtr
MidiMsg::
unparse() const
{
	bool is_ed = (msgt() == MidiMsg::system && smsgt() == smsgt::ext);
	UtilStringPtr ed_string;
	if (is_ed)
		ed_string = ed->unparse();
	char* str = new char[80 + ((ed_string == NULL) ? 0 : ed_string->length)];
	int len = sprintf(str, "%8ld ", time );
	if (is_ed)
		sprintf(str+len, "ed: %s", (const char*)(*ed_string));
	else
		sprintf(str+len, "%X %X %X ", s, d1, d2);
	UtilStringPtr result = new UtilString(str);
	delete[] str;
	return result;
}
		
void
MidiMsg::
sanityCheck() const
{
	assert (s > 127);
	assert (d1 < 128);
	assert (d2 < 128);
	if (ed != NULL)
	{
		assert(msgt() == system);
		assert(smsgt() == smsgt::ext);
	}
	else
		assert(msgt() != system || smsgt() != smsgt::ext);
}

bool
MidiMsg::
operator==(const MidiMsg& m) const
{
	if (time != m.time)
		return 0;
	bool is_ed = (msgt() == MidiMsg::system && smsgt() == smsgt::ext);
	if (is_ed)
		return ((*(unparse())) == (*(m.unparse())));
	return (s == m.s && d1 == m.d1 && d2 == m.d2 && node == m.node);
}

void
MidiMsg::
set( uchar msgt, uchar lsn, uchar dat1, uchar dat2, ioref in_node )
{
	set_stat_byt( msgt, lsn ); // sets s
	d1 = dat1;
	d2 = dat2;
	node = in_node;
}

bool
MidiMsg::
is_on_for( const MidiMsg& m ) const
{
	return m.is_note_end()
		? ( node == m.node && chan() == m.chan() && key() == m.key() )
		: false;
}


//--------------------------------------------------------------------------------------------------

int
ext_data::
sprint(char *s) const
{
	int len;
	if (st==smsgt::file_meta) len = meta_event.sprint(s);
	else len = *s = 0;
	return len;
}

UtilStringPtr
ext_data::
unparse() const
{
	if (st==smsgt::file_meta)
		return meta_event.unparse();
	else
		return new UtilString; // empty string
}

//--------------------------------------------------------------------------------------------------

/*
void
baws::
init( const uchar *r, long s )
{
	d = new UtilArray<uchar>(s);
	for (int i=0; i<s; i++) (*d)[i] = r[i];
}
*/

void
baws::
init( const uchar *r, long s )
{
	if (d)
		delete[] d;
	l = s;
	d = new uchar[s];
	for (int i=0; i<s; i++) d[i] = r[i];
}

//--------------------------------------------------------------------------------------------------

void
meta::
init( int meta_ty, const uchar *r, long s )
{
	mt = uchar(meta_ty);
	switch (mt)
	{
		case sequence_num:	seq_num = (int2)swap2((uint2)(*( (int2*) r ))); break;	
		case txt_generic:
		case txt_copyright:
		case txt_seq_or_trk:
		case txt_instrument:
		case txt_lyric:
		case txt_marker:
		case txt_cue:
		{
			char* tmp = new char[s+1];
			for (int i=0; i<s; i++) tmp[i] = r[i];
			tmp[s] = 0;
			txt = new UtilString(tmp);
			delete[] tmp;
			break;
		}
		case chan_prefix:		ch_prefix = *r; break;
		case trk_end:			break;
		case tempo:				tem = (((long) r[0]) << 16) + (((long) r[1]) << 8) + r[2]; break;
//		case tempo:				tem = (((long) r[0]) << 2) + (((long) r[1]) << 1) + r[2]; break;
		case SMPTE_offset:
			o.format  = (offs::fmt) ( (r[0] & 0x60) >> 5 );
			o.hr      =  uchar( r[0] & 0x1F );
			o.min = r[1]; o.sec = r[2]; o.fr = r[3]; o.centifr = r[4];
			break;
		case time_sig:
			t.num = r[0];
			t.denom	= uchar( 1 << r[1] );
			t.Mclks_p_beat = r[2]; t.not32s_p_24_Mclks = r[3];
			break;
		case key_sig:
		{
			char sr = (signed char) r[0];
			k.m = (ksig::mod) ( sr < 0 ? ksig::flat : ksig::sharp );
			k.n = uchar( sr < 0 ? -sr : sr );
			k.q = (ksig::qual) r[1];
			break;
		}
		case sequencer_spec: seq_spec.init(r, s);
	}
}

int
meta::
sprint( char *s ) const
{
	switch ( mt )
	{
	case sequence_num:	return sprintf( s, "sequence # %d", seq_num);
	case txt_generic:		return sprintf( s, "text event '%s'", (const char*)(*txt));
	case txt_copyright:	return sprintf( s, "copyright notice '%s'", (const char*)(*txt));
	case txt_seq_or_trk:	return sprintf( s, "sequence or track name '%s'", (const char*)(*txt));
	case txt_instrument:	return sprintf( s, "instrument name '%s'", (const char*)(*txt));
	case txt_lyric: 		return sprintf( s, "lyric '%s'", (const char*)(*txt));
	case txt_marker:		return sprintf( s, "marker '%s'", (const char*)(*txt));
	case txt_cue: 			return sprintf( s, "cue point '%s'", (const char*)(*txt));
	case chan_prefix: 	return sprintf( s, "channel prefix %d", ch_prefix);
	case trk_end: 			return sprintf( s, "end of track");
	case tempo: 			return sprintf( s, "%ld microseconds per MIDI quarter note", tem);
	case SMPTE_offset:	return
								sprintf( s, "SMPTE offset: %d frames/sec %s, %02d:%02d:%02d:%02d.%02d",
												o.format == offs::f24 ? 24 : o.format == offs::f25 ? 25 : 30,
												o.format == offs::f30d ? "drop" : "non-drop",
												o.hr, o.min, o.sec, o.fr, o.centifr );
	case time_sig: 		return
								sprintf( s,	"time signature %d/%d, %d MIDI clocks per beat, "
												"_%d_ notated 32nd notes per 24 MIDI clocks",
												t.num, t.denom, t.Mclks_p_beat, t.not32s_p_24_Mclks );
	case key_sig: 			return
								sprintf( s, "key signature: %d %s(s), %s",
												k.n, k.m == ksig::flat ? "flat" : "sharp",
												k.q == ksig::major ? "major" : "minor" );
												
	case sequencer_spec:	return sprintf( s, "sequencer specific");
	}
	return 0;
}

UtilStringPtr
meta::
unparse() const
{
	char* s = new char[256 + ((txt == NULL) ? 0 : txt->length)];
	sprint(s);
	UtilStringPtr result = new UtilString(s);
	delete[] s;
	return result;
}


// Author: Ben Denckla
}